@@ -1,11 +1,11 @@
 /*-
  ***********************************************************************
  *
- * $Id: pad-binary-guts.c,v 1.2 2004/03/13 17:26:28 mavrik Exp $
+ * $Id: pad-binary-guts.c,v 1.15 2012/01/07 07:56:14 mavrik Exp $
  *
  ***********************************************************************
  *
- * Copyright 2002-2004 The WebJob Project, All Rights Reserved.
+ * Copyright 2002-2012 The WebJob Project, All Rights Reserved.
  *
  ***********************************************************************
  */
@@ -32,6 +32,7 @@ PAD_CALL_TABLE gsCallTable[] =
   { PAD_VOID_CALL PaDExtractPayload, "PaDExtractPayload()" },
   { PAD_VOID_CALL PaDGetBasename, "PaDGetBasename()" },
   { PAD_VOID_CALL PaDGetCallname, "PaDGetCallname()" },
+  { PAD_VOID_CALL PaDGetEnvValue, "PaDGetEnvValue()" },
   { PAD_VOID_CALL PaDGetMyHandle, "PaDGetMyHandle()" },
   { PAD_VOID_CALL PaDGetPropertiesReference, "PaDGetPropertiesReference()" },
   { PAD_VOID_CALL PaDLocateDelimiter, "PaDLocateDelimiter()" },
@@ -140,14 +141,13 @@ main(int iArgumentCount, char *ppcArgumentVector[])
    *
    *********************************************************************
    */
-  if (!PAD_MUTE)
+  if (psProperties->iMute == PAD_MUTE_OFF)
   {
     fprintf(stderr, "Extracting payload...\n");
   }
   iError = PaDExtractPayload(psProperties, acLocalError);
   if (iError != ER_OK)
   {
-    psProperties->iUnlinkPayload = 1;
     fprintf(stderr, "%s: %s\n", PaDGetCallname(pvCall), acLocalError);
     PaDShutdown(XER_Extract);
   }
@@ -163,7 +163,7 @@ main(int iArgumentCount, char *ppcArgumentVector[])
   {
     psProperties->iUnlinkPayload = 1;
     iNeedXBit = PaDBuildCmdLine(iArgumentCount, ppcArgumentVector, psProperties->pcPayload, psProperties->pcCmdLine);
-    if (!PAD_MUTE)
+    if (psProperties->iMute == PAD_MUTE_OFF)
     {
       fprintf(stderr, "Delivering payload... %s\n", psProperties->pcCmdLine);
     }
@@ -173,7 +173,7 @@ main(int iArgumentCount, char *ppcArgumentVector[])
       fprintf(stderr, "%s: %s\n", PaDGetCallname(pvCall), acLocalError);
       PaDShutdown(XER_Deliver);
     }
-    if (!PAD_MUTE)
+    if (psProperties->iMute == PAD_MUTE_OFF)
     {
       fprintf(stderr, "DeliveryStatus='%d'\n", iError);
     }
@@ -204,7 +204,12 @@ PaDBootStrap(char *pcError)
 {
   void              (*pvCall)() = PAD_VOID_CALL PaDBootStrap;
   char                acLocalError[MESSAGE_SIZE];
+  char               *pcValue = NULL;
   PAD_GUTS_PROPERTIES *psProperties;
+#ifdef UNIX
+  int                 i = 0;
+  int                 iLength = 0;
+#endif
 
 #ifdef WIN32
   /*-
@@ -246,6 +251,42 @@ PaDBootStrap(char *pcError)
     return ER;
   }
 
+  /*-
+   *********************************************************************
+   *
+   * Pull in envrionment variables.
+   *
+   *********************************************************************
+   */
+  pcValue = PaDGetEnvValue("PAD_MUTE");
+  psProperties->iMute = (pcValue == NULL) ? PAD_MUTE_OFF : (strcmp(pcValue, "0") == 0) ? PAD_MUTE_OFF : PAD_MUTE_ON;
+
+  pcValue = PaDGetEnvValue("PAD_OVERWRITE");
+  psProperties->iOverwritePayload = (pcValue == NULL) ? PAD_OVERWRITE_ON : (strcmp(pcValue, "0") == 0) ? PAD_OVERWRITE_OFF : PAD_OVERWRITE_ON;
+
+#ifdef UNIX
+  pcValue = PaDGetEnvValue("PAD_UMASK");
+  if (pcValue != NULL && (iLength = strlen(pcValue)) <= 4)
+  {
+    for (i = 0; i < iLength; i++)
+    {
+      if (!isdigit((int)pcValue[i]) || pcValue[i] > '7') /* Accept only octal digits. */
+      {
+        break;
+      }
+    }
+    if (i == iLength) /* We have 1-4 octal digits, so strtoul() should succeed. */
+    {
+      psProperties->ulUmask = strtoul(pcValue, NULL, 8);
+    }
+  }
+  else
+  {
+    psProperties->ulUmask = PAD_UMASK;
+  }
+  umask((mode_t)psProperties->ulUmask);
+#endif
+
   return ER_OK;
 }
 
@@ -459,20 +500,25 @@ PaDExtractPayload(PAD_GUTS_PROPERTIES *psProperties, char *pcError)
   FILE               *pFileFrom;
   FILE               *pFileTo;
   int                 iError;
+  struct stat         statEntry;
 
-#ifdef UNIX
-  umask(022);
-#endif
+  if (stat(psProperties->pcPayload, &statEntry) == 0 && psProperties->iOverwritePayload == 0)
+  {
+    snprintf(pcError, MESSAGE_SIZE, "%s: stat(): File='%s' Error='File exists and PAD_OVERWRITE is disabled. Extraction aborted.'", PaDGetCallname(pvCall), psProperties->pcPayload);
+    return ER;
+  }
   pFileFrom = psProperties->pFilePaD;
   pFileTo = fopen(psProperties->pcPayload, "wb");
   if (pFileTo == NULL)
   {
+    psProperties->iUnlinkPayload = 1;
     snprintf(pcError, MESSAGE_SIZE, "%s: fopen(): File='%s' Error='%s'", PaDGetCallname(pvCall), psProperties->pcPayload, strerror(errno));
     return ER;
   }
   iError = PaDReadWrite(pFileFrom, pFileTo, acLocalError);
   if (iError != ER_OK)
   {
+    psProperties->iUnlinkPayload = 1;
     snprintf(pcError, MESSAGE_SIZE, "%s: %s:", PaDGetCallname(pvCall), acLocalError);
     fclose(pFileTo);
     return ER;
