@@ -1,14 +1,16 @@
 /*-
  ***********************************************************************
  *
- * $Id: all-includes.h,v 1.5 2004/03/06 20:45:50 mavrik Exp $
+ * $Id: all-includes.h,v 1.15 2012/01/07 07:56:14 mavrik Exp $
  *
  ***********************************************************************
  *
- * Copyright 2001-2004 Klayton Monroe, All Rights Reserved.
+ * Copyright 2001-2012 The WebJob Project, All Rights Reserved.
  *
  ***********************************************************************
  */
+#ifndef _ALL_INCLUDES_H_INCLUDED
+#define _ALL_INCLUDES_H_INCLUDED
 
 #if defined(UNIX) || defined(MINGW32)
 #include "config.h"
@@ -16,3 +18,5 @@
 
 #include "sys-includes.h"
 #include "app-includes.h"
+
+#endif /* !_ALL_INCLUDES_H_INCLUDED */
