@@ -1,11 +1,11 @@
 /*-
  ***********************************************************************
  *
- * $Id: properties.c,v 1.19 2004/04/09 17:22:26 mavrik Exp $
+ * $Id: properties.c,v 1.45 2012/01/07 07:56:14 mavrik Exp $
  *
  ***********************************************************************
  *
- * Copyright 2001-2004 Klayton Monroe, All Rights Reserved.
+ * Copyright 2001-2012 The WebJob Project, All Rights Reserved.
  *
  ***********************************************************************
  */
@@ -68,7 +68,7 @@ int
 PropertiesReadFile(char *pcFilename, WEBJOB_PROPERTIES *psProperties, char *pcError)
 {
   const char          acRoutine[] = "PropertiesReadFile()";
-  char                acLocalError[MESSAGE_SIZE] = { 0 };
+  char                acLocalError[MESSAGE_SIZE] = "";
   char                acLine[PROPERTIES_MAX_LINE_LENGTH];
   char               *pc;
   int                 iError;
@@ -76,9 +76,7 @@ PropertiesReadFile(char *pcFilename, WEBJOB_PROPERTIES *psProperties, char *pcEr
   int                 iLineNumber;
   int                 iSaveLength;
   FILE               *pFile;
-#ifdef UNIX
   struct stat         statEntry;
-#endif
 
   /*-
    *********************************************************************
@@ -99,20 +97,34 @@ PropertiesReadFile(char *pcFilename, WEBJOB_PROPERTIES *psProperties, char *pcEr
   }
   else
   {
-#ifdef UNIX
-    iError = lstat(pcFilename, &statEntry);
+    iError = stat(pcFilename, &statEntry);
     if (iError == -1)
     {
+      /*-
+       *****************************************************************
+       *
+       * Return OK if the specified config file (Import) doesn't exist.
+       * By making this allowance, an admin can (more easily) create a
+       * generic config file that spans multiple clients or an entire
+       * WebJob deployment. If the recursion level is zero, then we are
+       * dealing with a command line config file, and in that case, the
+       * file must exist.
+       *
+       *****************************************************************
+       */
+      if (psProperties->iImportRecursionLevel > 0 && errno == ENOENT)
+      {
+        return ER_OK;
+      }
       snprintf(pcError, MESSAGE_SIZE, "%s: File = [%s]: %s", acRoutine, pcFilename, strerror(errno));
       return ER_lstat;
     }
 
-    if (!S_ISREG(statEntry.st_mode))
+    if (!((statEntry.st_mode & S_IFMT) == S_IFREG))
     {
       snprintf(pcError, MESSAGE_SIZE, "%s: File = [%s]: A regular file is required.", acRoutine, pcFilename);
       return ER_BadValue;
     }
-#endif
 
     if ((pFile = fopen(pcFilename, "r")) == NULL)
     {
@@ -195,7 +207,7 @@ PropertiesReadFile(char *pcFilename, WEBJOB_PROPERTIES *psProperties, char *pcEr
      *
      *******************************************************************
      */
-    while (isspace((int) acLine[iLength - 1]))
+    while (iLength > 0 && isspace((int) acLine[iLength - 1]))
     {
       acLine[iLength--] = 0;
     }
@@ -250,7 +262,7 @@ int
 PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
 {
   const char          acRoutine[] = "PropertiesReadLine()";
-  char                acLocalError[MESSAGE_SIZE] = { 0 };
+  char                acLocalError[MESSAGE_SIZE] = "";
   char               *pc;
   char               *pcControl;
   char               *pcEnd;
@@ -305,11 +317,14 @@ PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
    *********************************************************************
    */
   iLength = strlen(pc);
-  pcEnd = &pc[iLength - 1];
-  while (isspace((int) *pcEnd))
+  if (iLength > 0)
   {
-    *pcEnd-- = 0;
-    iLength--;
+    pcEnd = &pc[iLength - 1];
+    while (iLength > 0 && isspace((int) *pcEnd))
+    {
+      *pcEnd-- = 0;
+      iLength--;
+    }
   }
 
   /*-
@@ -331,10 +346,15 @@ PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
    *
    *********************************************************************
    */
-  pcEnd = &pcControl[strlen(pcControl) - 1];
-  while (isspace((int) *pcEnd))
+  iLength = strlen(pcControl);
+  if (iLength > 0)
   {
-    *pcEnd-- = 0;
+    pcEnd = &pcControl[iLength - 1];
+    while (iLength > 0 && isspace((int) *pcEnd))
+    {
+      *pcEnd-- = 0;
+      iLength--;
+    }
   }
 
   /*-
@@ -346,6 +366,7 @@ PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
    *
    *********************************************************************
    */
+  iLength = strlen(pc);
 
   iRunMode = psProperties->iRunMode;
 
@@ -364,9 +385,21 @@ PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
       psProperties->iHashSize = SHA1_HASH_SIZE;
       psProperties->piHashStream = SHA1HashStream;
     }
+    else if (strcasecmp(pc, WEBJOB_HASHTYPE_SHA256) == 0)
+    {
+      strncpy(psProperties->acHashType, WEBJOB_HASHTYPE_SHA256, WEBJOB_HASHTYPE_SIZE);
+      psProperties->iHashSize = SHA256_HASH_SIZE;
+      psProperties->piHashStream = SHA256HashStream;
+    }
     else
     {
-      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Value must be [%s|%s].", acRoutine, pcControl, WEBJOB_HASHTYPE_MD5, WEBJOB_HASHTYPE_SHA1);
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Value must be [%s|%s|%s].",
+        acRoutine,
+        pcControl,
+        WEBJOB_HASHTYPE_MD5,
+        WEBJOB_HASHTYPE_SHA1,
+        WEBJOB_HASHTYPE_SHA256
+        );
       return ER_BadValue;
     }
     pcTempHash = realloc(psProperties->pcStdOutHash, psProperties->iHashSize);
@@ -407,6 +440,18 @@ PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
     psProperties->sFound.OverwriteExecutableFound = TRUE;
   }
 
+  else if (strcasecmp(pcControl, KEY_Priority) == 0 && RUN_MODE_IS_SET(MODES_Priority, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.PriorityFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_PRIORITY_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
+    strncpy(psProperties->acPriority, pc, WEBJOB_MAX_PRIORITY_LENGTH);
+    psProperties->sFound.PriorityFound = TRUE;
+  }
+
   else if (strcasecmp(pcControl, KEY_RunType) == 0 && RUN_MODE_IS_SET(MODES_RunType, iRunMode))
   {
     DUPLICATE_ERROR(psProperties->sFound.RunTypeFound);
@@ -426,6 +471,11 @@ PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
     psProperties->sFound.RunTypeFound = TRUE;
   }
 
+  else if (strcasecmp(pcControl, KEY_StrictControls) == 0 && RUN_MODE_IS_SET(MODES_StrictControls, iRunMode))
+  {
+    EVALUATE_TWOSTATE(pc, "y", "n", psProperties->iStrictControls);
+  }
+
   else if (strcasecmp(pcControl, KEY_TempDirectory) == 0 && RUN_MODE_IS_SET(MODES_TempDirectory, iRunMode))
   {
     DUPLICATE_ERROR(psProperties->sFound.TempDirectoryFound);
@@ -438,6 +488,38 @@ PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
     psProperties->sFound.TempDirectoryFound = TRUE;
   }
 
+#ifdef UNIX
+  else if (strcasecmp(pcControl, KEY_TimeoutSignal) == 0 && RUN_MODE_IS_SET(MODES_TimeoutSignal, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.TimeoutSignalFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_32BIT_NUMBER_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
+    while (iLength > 0)
+    {
+      if (!isdigit((int) pc[iLength - 1]))
+      {
+        snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Value = [%s], Value must be %d-%d.", acRoutine, pcControl, pc, WEBJOB_MIN_SIGNAL, WEBJOB_MAX_SIGNAL);
+        return ER_BadValue;
+      }
+      iLength--;
+    }
+    iValue = atoi(pc);
+    if (iValue < WEBJOB_MIN_SIGNAL || iValue > WEBJOB_MAX_SIGNAL)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Value = [%d], Value must be %d-%d.", acRoutine, pcControl, iValue, WEBJOB_MIN_SIGNAL, WEBJOB_MAX_SIGNAL);
+      return ER_BadValue;
+    }
+    else
+    {
+      psProperties->iTimeoutSignal = iValue;
+    }
+    psProperties->sFound.TimeoutSignalFound = TRUE;
+  }
+#endif
+
   else if (strcasecmp(pcControl, KEY_UnlinkExecutable) == 0 && RUN_MODE_IS_SET(MODES_UnlinkExecutable, iRunMode))
   {
     DUPLICATE_ERROR(psProperties->sFound.UnlinkExecutableFound);
@@ -452,111 +534,276 @@ PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
     psProperties->sFound.UnlinkOutputFound = TRUE;
   }
 
-  else if (strcasecmp(pcControl, KEY_URLAuthType) == 0 && RUN_MODE_IS_SET(MODES_URLAuthType, iRunMode))
+  else if (strcasecmp(pcControl, KEY_UploadOnGetFailure) == 0 && RUN_MODE_IS_SET(MODES_UploadOnGetFailure, iRunMode))
   {
-    DUPLICATE_ERROR(psProperties->sFound.URLAuthTypeFound);
+    DUPLICATE_ERROR(psProperties->sFound.UploadOnGetFailureFound);
+    EVALUATE_TWOSTATE(pc, "y", "n", psProperties->iUploadOnGetFailure);
+    psProperties->sFound.UploadOnGetFailureFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlAuthType) == 0 && RUN_MODE_IS_SET(MODES_UrlAuthType, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlAuthTypeFound);
     if (strcasecmp(pc, "basic") == 0)
     {
-      psProperties->iURLAuthType = HTTP_AUTH_TYPE_BASIC;
+      psProperties->iUrlAuthType = HTTP_AUTH_TYPE_BASIC;
     }
     else if (strcasecmp(pc, "none") == 0)
     {
-      psProperties->iURLAuthType = HTTP_AUTH_TYPE_NONE;
+      psProperties->iUrlAuthType = HTTP_AUTH_TYPE_NONE;
     }
     else
     {
       snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Value must be [basic|none].", acRoutine, pcControl);
       return ER_BadValue;
     }
-    psProperties->sFound.URLAuthTypeFound = TRUE;
+    psProperties->sFound.UrlAuthTypeFound = TRUE;
   }
 
-  else if (strcasecmp(pcControl, KEY_URLGetURL) == 0 && RUN_MODE_IS_SET(MODES_URLGetURL, iRunMode))
+  else if (strcasecmp(pcControl, KEY_UrlGetProxyAuthType) == 0 && RUN_MODE_IS_SET(MODES_UrlGetProxyAuthType, iRunMode))
   {
-    DUPLICATE_ERROR(psProperties->sFound.URLGetURLFound);
-    if (iLength < 1 || iLength > WEBJOB_MAX_PATHNAME_LENGTH - 1)
+    DUPLICATE_ERROR(psProperties->sFound.UrlGetProxyAuthTypeFound);
+    if (strcasecmp(pc, "basic") == 0)
     {
-      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
-      return ER_Length;
+      psProperties->iUrlGetProxyAuthType = HTTP_AUTH_TYPE_BASIC;
     }
-    psProperties->psGetURL = HTTPParseURL(pc, acLocalError);
-    if (psProperties->psGetURL == NULL)
+    else if (strcasecmp(pc, "none") == 0)
     {
-      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s]: %s", acRoutine, pcControl, acLocalError);
-      return ER_HTTPParseURL;
-    }
-    psProperties->sFound.URLGetURLFound = TRUE;
-  }
-
-  else if (strcasecmp(pcControl, KEY_URLPutURL) == 0 && RUN_MODE_IS_SET(MODES_URLPutURL, iRunMode))
-  {
-    DUPLICATE_ERROR(psProperties->sFound.URLPutURLFound);
-    if (iLength < 1 || iLength > WEBJOB_MAX_PATHNAME_LENGTH - 1)
-    {
-      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
-      return ER_Length;
-    }
-    psProperties->psPutURL = HTTPParseURL(pc, acLocalError);
-    if (psProperties->psPutURL == NULL)
-    {
-      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s]: %s", acRoutine, pcControl, acLocalError);
-      return ER_HTTPParseURL;
-    }
-    psProperties->sFound.URLPutURLFound = TRUE;
-  }
-
-  else if (strcasecmp(pcControl, KEY_URLDownloadLimit) == 0 && RUN_MODE_IS_SET(MODES_URLDownloadLimit, iRunMode))
-  {
-    DUPLICATE_ERROR(psProperties->sFound.URLDownloadLimitFound);
-    while (iLength > 0)
-    {
-      if (!isdigit((int) pc[iLength - 1]))
-      {
-        snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Value = [%s], Value must be 1-%d.", acRoutine, pcControl, pc, WEBJOB_MAX_CONTENT_LENGTH);
-        return ER_BadValue;
-      }
-      iLength--;
-    }
-    iValue = atoi(pc);
-    if (iValue < 1 || iValue > WEBJOB_MAX_CONTENT_LENGTH)
-    {
-      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Value = [%s], Value must be 1-%d.", acRoutine, pcControl, pc, WEBJOB_MAX_CONTENT_LENGTH);
-      return ER_BadValue;
+      psProperties->iUrlGetProxyAuthType = HTTP_AUTH_TYPE_NONE;
     }
     else
     {
-      psProperties->iURLDownloadLimit = iValue;
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Value must be [basic|none].", acRoutine, pcControl);
+      return ER_BadValue;
     }
-    psProperties->sFound.URLDownloadLimitFound = TRUE;
+    psProperties->sFound.UrlGetProxyAuthTypeFound = TRUE;
   }
 
-  else if (strcasecmp(pcControl, KEY_URLUsername) == 0 && RUN_MODE_IS_SET(MODES_URLUsername, iRunMode))
+  else if (strcasecmp(pcControl, KEY_UrlGetProxyEnable) == 0 && RUN_MODE_IS_SET(MODES_UrlGetProxyEnable, iRunMode))
   {
-    DUPLICATE_ERROR(psProperties->sFound.URLUsernameFound);
+    DUPLICATE_ERROR(psProperties->sFound.UrlGetProxyEnableFound);
+    EVALUATE_TWOSTATE(pc, "y", "n", psProperties->iUrlGetProxyEnable);
+    psProperties->sFound.UrlGetProxyEnableFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlGetProxyHost) == 0 && RUN_MODE_IS_SET(MODES_UrlGetProxyHost, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlGetProxyHostFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_HOSTNAME_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
+    strncpy(psProperties->acUrlGetProxyHost, pc, WEBJOB_MAX_HOSTNAME_LENGTH);
+    psProperties->sFound.UrlGetProxyHostFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlGetProxyPassword) == 0 && RUN_MODE_IS_SET(MODES_UrlGetProxyPassword, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlGetProxyPasswordFound);
+    psProperties->pcUrlGetProxyPassword = WebJobDecodeValue(pc, acLocalError);
+    if (psProperties->pcUrlGetProxyPassword == NULL)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s]: %s", acRoutine, pcControl, acLocalError);
+      return ER;
+    }
+    psProperties->sFound.UrlGetProxyPasswordFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlGetProxyPort) == 0 && RUN_MODE_IS_SET(MODES_UrlGetProxyPort, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlGetProxyPortFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_PORT_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
+    strncpy(psProperties->acUrlGetProxyPort, pc, WEBJOB_MAX_PORT_LENGTH);
+    psProperties->sFound.UrlGetProxyPortFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlGetProxyUsername) == 0 && RUN_MODE_IS_SET(MODES_UrlGetProxyUsername, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlGetProxyUsernameFound);
     if (iLength < 1 || iLength > WEBJOB_MAX_USERNAME_LENGTH - 1)
     {
       snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
       return ER_Length;
     }
-    strncpy(psProperties->acURLUsername, pc, WEBJOB_MAX_USERNAME_LENGTH);
-    psProperties->sFound.URLUsernameFound = TRUE;
+    strncpy(psProperties->acUrlGetProxyUsername, pc, WEBJOB_MAX_USERNAME_LENGTH);
+    psProperties->sFound.UrlGetProxyUsernameFound = TRUE;
   }
 
-  else if (strcasecmp(pcControl, KEY_URLPassword) == 0 && RUN_MODE_IS_SET(MODES_URLPassword, iRunMode))
+  else if (strcasecmp(pcControl, KEY_UrlPutProxyAuthType) == 0 && RUN_MODE_IS_SET(MODES_UrlPutProxyAuthType, iRunMode))
   {
-    DUPLICATE_ERROR(psProperties->sFound.URLPasswordFound);
-    if (iLength < 1 || iLength > WEBJOB_MAX_PASSWORD_LENGTH - 1)
+    DUPLICATE_ERROR(psProperties->sFound.UrlPutProxyAuthTypeFound);
+    if (strcasecmp(pc, "basic") == 0)
+    {
+      psProperties->iUrlPutProxyAuthType = HTTP_AUTH_TYPE_BASIC;
+    }
+    else if (strcasecmp(pc, "none") == 0)
+    {
+      psProperties->iUrlPutProxyAuthType = HTTP_AUTH_TYPE_NONE;
+    }
+    else
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Value must be [basic|none].", acRoutine, pcControl);
+      return ER_BadValue;
+    }
+    psProperties->sFound.UrlPutProxyAuthTypeFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlPutProxyEnable) == 0 && RUN_MODE_IS_SET(MODES_UrlPutProxyEnable, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlPutProxyEnableFound);
+    EVALUATE_TWOSTATE(pc, "y", "n", psProperties->iUrlPutProxyEnable);
+    psProperties->sFound.UrlPutProxyEnableFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlPutProxyHost) == 0 && RUN_MODE_IS_SET(MODES_UrlPutProxyHost, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlPutProxyHostFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_HOSTNAME_LENGTH - 1)
     {
       snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
       return ER_Length;
     }
-    strncpy(psProperties->acURLPassword, pc, WEBJOB_MAX_PASSWORD_LENGTH);
-    psProperties->sFound.URLPasswordFound = TRUE;
+    strncpy(psProperties->acUrlPutProxyHost, pc, WEBJOB_MAX_HOSTNAME_LENGTH);
+    psProperties->sFound.UrlPutProxyHostFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlPutProxyPassword) == 0 && RUN_MODE_IS_SET(MODES_UrlPutProxyPassword, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlPutProxyPasswordFound);
+    psProperties->pcUrlPutProxyPassword = WebJobDecodeValue(pc, acLocalError);
+    if (psProperties->pcUrlPutProxyPassword == NULL)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s]: %s", acRoutine, pcControl, acLocalError);
+      return ER;
+    }
+    psProperties->sFound.UrlPutProxyPasswordFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlPutProxyPort) == 0 && RUN_MODE_IS_SET(MODES_UrlPutProxyPort, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlPutProxyPortFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_PORT_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
+    strncpy(psProperties->acUrlPutProxyPort, pc, WEBJOB_MAX_PORT_LENGTH);
+    psProperties->sFound.UrlPutProxyPortFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlPutProxyUsername) == 0 && RUN_MODE_IS_SET(MODES_UrlPutProxyUsername, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlPutProxyUsernameFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_USERNAME_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
+    strncpy(psProperties->acUrlPutProxyUsername, pc, WEBJOB_MAX_USERNAME_LENGTH);
+    psProperties->sFound.UrlPutProxyUsernameFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlGetUrl) == 0 && RUN_MODE_IS_SET(MODES_UrlGetUrl, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlGetUrlFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_PATHNAME_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
+    psProperties->psGetUrl = HttpParseUrl(pc, acLocalError);
+    if (psProperties->psGetUrl == NULL)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s]: %s", acRoutine, pcControl, acLocalError);
+      return ER_HttpParseUrl;
+    }
+    psProperties->sFound.UrlGetUrlFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlPutUrl) == 0 && RUN_MODE_IS_SET(MODES_UrlPutUrl, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlPutUrlFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_PATHNAME_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
+    psProperties->psPutUrl = HttpParseUrl(pc, acLocalError);
+    if (psProperties->psPutUrl == NULL)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s]: %s", acRoutine, pcControl, acLocalError);
+      return ER_HttpParseUrl;
+    }
+    psProperties->sFound.UrlPutUrlFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlDownloadLimit) == 0 && RUN_MODE_IS_SET(MODES_UrlDownloadLimit, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlDownloadLimitFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_32BIT_NUMBER_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
+    while (iLength > 0)
+    {
+      if (!isdigit((int) pc[iLength - 1]))
+      {
+        snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Value = [%s], Value must be 0-%d.", acRoutine, pcControl, pc, WEBJOB_MAX_CONTENT_LENGTH);
+        return ER_BadValue;
+      }
+      iLength--;
+    }
+    iValue = atoi(pc);
+    if (iValue < 0 || iValue > WEBJOB_MAX_CONTENT_LENGTH)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Value = [%s], Value must be 0-%d.", acRoutine, pcControl, pc, WEBJOB_MAX_CONTENT_LENGTH);
+      return ER_BadValue;
+    }
+    else
+    {
+      psProperties->iUrlDownloadLimit = iValue;
+    }
+    psProperties->sFound.UrlDownloadLimitFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlUsername) == 0 && RUN_MODE_IS_SET(MODES_UrlUsername, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlUsernameFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_USERNAME_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
+    strncpy(psProperties->acUrlUsername, pc, WEBJOB_MAX_USERNAME_LENGTH);
+    psProperties->sFound.UrlUsernameFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_UrlPassword) == 0 && RUN_MODE_IS_SET(MODES_UrlPassword, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.UrlPasswordFound);
+    psProperties->pcUrlPassword = WebJobDecodeValue(pc, acLocalError);
+    if (psProperties->pcUrlPassword == NULL)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s]: %s", acRoutine, pcControl, acLocalError);
+      return ER;
+    }
+    psProperties->sFound.UrlPasswordFound = TRUE;
   }
 
   else if (strcasecmp(pcControl, KEY_GetTimeLimit) == 0 && RUN_MODE_IS_SET(MODES_GetTimeLimit, iRunMode))
   {
     DUPLICATE_ERROR(psProperties->sFound.GetTimeLimitFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_32BIT_NUMBER_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
     while (iLength > 0)
     {
       if (!isdigit((int) pc[iLength - 1]))
@@ -582,6 +829,11 @@ PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
   else if (strcasecmp(pcControl, KEY_PutTimeLimit) == 0 && RUN_MODE_IS_SET(MODES_PutTimeLimit, iRunMode))
   {
     DUPLICATE_ERROR(psProperties->sFound.PutTimeLimitFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_32BIT_NUMBER_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
     while (iLength > 0)
     {
       if (!isdigit((int) pc[iLength - 1]))
@@ -607,6 +859,11 @@ PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
   else if (strcasecmp(pcControl, KEY_RunTimeLimit) == 0 && RUN_MODE_IS_SET(MODES_RunTimeLimit, iRunMode))
   {
     DUPLICATE_ERROR(psProperties->sFound.RunTimeLimitFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_32BIT_NUMBER_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
     while (iLength > 0)
     {
       if (!isdigit((int) pc[iLength - 1]))
@@ -630,55 +887,68 @@ PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
   }
 
 #ifdef USE_SSL
-  else if (strcasecmp(pcControl, KEY_SSLBundledCAsFile) == 0 && RUN_MODE_IS_SET(MODES_SSLBundledCAsFile, iRunMode))
+  else if (strcasecmp(pcControl, KEY_SslBundledCAsFile) == 0 && RUN_MODE_IS_SET(MODES_SslBundledCAsFile, iRunMode))
   {
-    DUPLICATE_ERROR(psProperties->sFound.SSLBundledCAsFileFound);
+    DUPLICATE_ERROR(psProperties->sFound.SslBundledCAsFileFound);
     if (iLength < 1 || iLength > WEBJOB_MAX_PATHNAME_LENGTH - 1)
     {
       snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
       return ER_Length;
     }
-    iError = SSLSetBundledCAsFile(psProperties->psSSLProperties, pc, acLocalError);
+    iError = SslSetBundledCAsFile(psProperties->psSslProperties, pc, acLocalError);
     if (iError != ER_OK)
     {
       snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s]: %s", acRoutine, pcControl, acLocalError);
       return iError;
     }
-    psProperties->sFound.SSLBundledCAsFileFound = TRUE;
+    psProperties->sFound.SslBundledCAsFileFound = TRUE;
   }
 
-  else if (strcasecmp(pcControl, KEY_SSLPassPhrase) == 0 && RUN_MODE_IS_SET(MODES_SSLPassPhrase, iRunMode))
+  else if (strcasecmp(pcControl, KEY_SslPassPhrase) == 0 && RUN_MODE_IS_SET(MODES_SslPassPhrase, iRunMode))
   {
-    DUPLICATE_ERROR(psProperties->sFound.SSLPassPhraseFound);
-    iError = SSLSetPassPhrase(psProperties->psSSLProperties, pc, acLocalError);
+    DUPLICATE_ERROR(psProperties->sFound.SslPassPhraseFound);
+    pc = WebJobDecodeValue(pc, acLocalError);
+    if (pc == NULL)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s]: %s", acRoutine, pcControl, acLocalError);
+      return ER;
+    }
+    iError = SslSetPassPhrase(psProperties->psSslProperties, pc, acLocalError);
     if (iError != ER_OK)
     {
       snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s]: %s", acRoutine, pcControl, acLocalError);
+      free(pc);
       return iError;
     }
-    psProperties->sFound.SSLPassPhraseFound = TRUE;
+    free(pc);
+    psProperties->sFound.SslPassPhraseFound = TRUE;
   }
 
-  else if (strcasecmp(pcControl, KEY_SSLPublicCertFile) == 0 && RUN_MODE_IS_SET(MODES_SSLPublicCertFile, iRunMode))
+  else if (strcasecmp(pcControl, KEY_SslPublicCertFile) == 0 && RUN_MODE_IS_SET(MODES_SslPublicCertFile, iRunMode))
   {
-    DUPLICATE_ERROR(psProperties->sFound.SSLPublicCertFileFound);
+    DUPLICATE_ERROR(psProperties->sFound.SslPublicCertFileFound);
     if (iLength < 1 || iLength > WEBJOB_MAX_PATHNAME_LENGTH - 1)
     {
       snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
       return ER_Length;
     }
-    iError = SSLSetPublicCertFile(psProperties->psSSLProperties, pc, acLocalError);
+    iError = SslSetPublicCertFile(psProperties->psSslProperties, pc, acLocalError);
     if (iError != ER_OK)
     {
       snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s]: %s", acRoutine, pcControl, acLocalError);
       return iError;
     }
-    psProperties->sFound.SSLPublicCertFileFound = TRUE;
+    psProperties->sFound.SslPublicCertFileFound = TRUE;
   }
 
-  else if (strcasecmp(pcControl, KEY_SSLMaxChainLength) == 0 && RUN_MODE_IS_SET(MODES_SSLMaxChainLength, iRunMode))
+  else if (strcasecmp(pcControl, KEY_SslMaxChainLength) == 0 && RUN_MODE_IS_SET(MODES_SslMaxChainLength, iRunMode))
   {
-    DUPLICATE_ERROR(psProperties->sFound.SSLMaxChainLengthFound);
+    DUPLICATE_ERROR(psProperties->sFound.SslMaxChainLengthFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_32BIT_NUMBER_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
     while (iLength > 0)
     {
       if (!isdigit((int) pc[iLength - 1]))
@@ -696,52 +966,52 @@ PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
     }
     else
     {
-      psProperties->psSSLProperties->iMaxChainLength = iValue;
+      psProperties->psSslProperties->iMaxChainLength = iValue;
     }
-    psProperties->sFound.SSLMaxChainLengthFound = TRUE;
+    psProperties->sFound.SslMaxChainLengthFound = TRUE;
   }
 
-  else if (strcasecmp(pcControl, KEY_SSLPrivateKeyFile) == 0 && RUN_MODE_IS_SET(MODES_SSLPrivateKeyFile, iRunMode))
+  else if (strcasecmp(pcControl, KEY_SslPrivateKeyFile) == 0 && RUN_MODE_IS_SET(MODES_SslPrivateKeyFile, iRunMode))
   {
-    DUPLICATE_ERROR(psProperties->sFound.SSLPrivateKeyFileFound);
+    DUPLICATE_ERROR(psProperties->sFound.SslPrivateKeyFileFound);
     if (iLength < 1 || iLength > WEBJOB_MAX_PATHNAME_LENGTH - 1)
     {
       snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
       return ER_Length;
     }
-    iError = SSLSetPrivateKeyFile(psProperties->psSSLProperties, pc, acLocalError);
+    iError = SslSetPrivateKeyFile(psProperties->psSslProperties, pc, acLocalError);
     if (iError != ER_OK)
     {
       snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s]: %s", acRoutine, pcControl, acLocalError);
       return iError;
     }
-    psProperties->sFound.SSLPrivateKeyFileFound = TRUE;
+    psProperties->sFound.SslPrivateKeyFileFound = TRUE;
   }
 
-  else if (strcasecmp(pcControl, KEY_SSLExpectedPeerCN) == 0 && RUN_MODE_IS_SET(MODES_SSLExpectedPeerCN, iRunMode))
+  else if (strcasecmp(pcControl, KEY_SslExpectedPeerCN) == 0 && RUN_MODE_IS_SET(MODES_SslExpectedPeerCN, iRunMode))
   {
-    DUPLICATE_ERROR(psProperties->sFound.SSLExpectedPeerCNFound);
-    iError = SSLSetExpectedPeerCN(psProperties->psSSLProperties, pc, acLocalError);
+    DUPLICATE_ERROR(psProperties->sFound.SslExpectedPeerCNFound);
+    iError = SslSetExpectedPeerCN(psProperties->psSslProperties, pc, acLocalError);
     if (iError != ER_OK)
     {
       snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s]: %s", acRoutine, pcControl, acLocalError);
       return iError;
     }
-    psProperties->sFound.SSLExpectedPeerCNFound = TRUE;
+    psProperties->sFound.SslExpectedPeerCNFound = TRUE;
   }
 
-  else if (strcasecmp(pcControl, KEY_SSLUseCertificate) == 0 && RUN_MODE_IS_SET(MODES_SSLUseCertificate, iRunMode))
+  else if (strcasecmp(pcControl, KEY_SslUseCertificate) == 0 && RUN_MODE_IS_SET(MODES_SslUseCertificate, iRunMode))
   {
-    DUPLICATE_ERROR(psProperties->sFound.SSLUseCertificateFound);
-    EVALUATE_TWOSTATE(pc, "y", "n", psProperties->psSSLProperties->iUseCertificate);
-    psProperties->sFound.SSLUseCertificateFound = TRUE;
+    DUPLICATE_ERROR(psProperties->sFound.SslUseCertificateFound);
+    EVALUATE_TWOSTATE(pc, "y", "n", psProperties->psSslProperties->iUseCertificate);
+    psProperties->sFound.SslUseCertificateFound = TRUE;
   }
 
-  else if (strcasecmp(pcControl, KEY_SSLVerifyPeerCert) == 0 && RUN_MODE_IS_SET(MODES_SSLVerifyPeerCert, iRunMode))
+  else if (strcasecmp(pcControl, KEY_SslVerifyPeerCert) == 0 && RUN_MODE_IS_SET(MODES_SslVerifyPeerCert, iRunMode))
   {
-    DUPLICATE_ERROR(psProperties->sFound.SSLVerifyPeerCertFound);
-    EVALUATE_TWOSTATE(pc, "y", "n", psProperties->psSSLProperties->iVerifyPeerCert);
-    psProperties->sFound.SSLVerifyPeerCertFound = TRUE;
+    DUPLICATE_ERROR(psProperties->sFound.SslVerifyPeerCertFound);
+    EVALUATE_TWOSTATE(pc, "y", "n", psProperties->psSslProperties->iVerifyPeerCert);
+    psProperties->sFound.SslVerifyPeerCertFound = TRUE;
   }
 #endif
 
@@ -773,6 +1043,11 @@ PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
   else if (strcasecmp(pcControl, KEY_GetHookStatus) == 0 && RUN_MODE_IS_SET(MODES_GetHookStatus, iRunMode))
   {
     DUPLICATE_ERROR(psProperties->sFound.GetHookStatusFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_32BIT_NUMBER_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
     while (iLength > 0)
     {
       if (!isdigit((int) pc[iLength - 1]))
@@ -819,10 +1094,34 @@ PropertiesReadLine(char *pcLine, WEBJOB_PROPERTIES *psProperties, char *pcError)
     psProperties->sFound.ClientIdFound = TRUE;
   }
 
+#ifdef USE_DSV
+  else if (strcasecmp(pcControl, KEY_DsvCertificateTree) == 0 && RUN_MODE_IS_SET(MODES_DsvCertificateTree, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.DsvCertificateTreeFound);
+    if (iLength < 1 || iLength > WEBJOB_MAX_PATHNAME_LENGTH - 1)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], Invalid length [%d].", acRoutine, pcControl, iLength);
+      return ER_Length;
+    }
+    strncpy(psProperties->acDsvCertificateTree, pc, WEBJOB_MAX_PATHNAME_LENGTH);
+    psProperties->sFound.DsvCertificateTreeFound = TRUE;
+  }
+
+  else if (strcasecmp(pcControl, KEY_DsvVerifySignature) == 0 && RUN_MODE_IS_SET(MODES_DsvVerifySignature, iRunMode))
+  {
+    DUPLICATE_ERROR(psProperties->sFound.DsvVerifySignatureFound);
+    EVALUATE_TWOSTATE(pc, "y", "n", psProperties->iDsvVerifySignature);
+    psProperties->sFound.DsvVerifySignatureFound = TRUE;
+  }
+#endif
+
   else
   {
-    snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], The specified control is not valid in this mode of operation.", acRoutine, pcLine);
-    return ER_Jibberish;
+    if (psProperties->iStrictControls)
+    {
+      snprintf(pcError, MESSAGE_SIZE, "%s: Control = [%s], The specified control is not valid in this mode of operation.", acRoutine, pcControl);
+      return ER_Jibberish;
+    }
   }
 
   return ER_OK;
