@@ -1,14 +1,23 @@
 /*-
  ***********************************************************************
  *
- * $Id: sys-includes.h,v 1.8 2004/03/06 20:45:50 mavrik Exp $
+ * $Id: sys-includes.h,v 1.20 2012/01/07 07:56:14 mavrik Exp $
  *
  ***********************************************************************
  *
- * Copyright 2001-2004 Klayton Monroe, All Rights Reserved.
+ * Copyright 2001-2012 The WebJob Project, All Rights Reserved.
  *
  ***********************************************************************
  */
+#ifndef _SYS_INCLUDES_H_INCLUDED
+#define _SYS_INCLUDES_H_INCLUDED
+
+#ifdef WEBJOB_SOLARIS
+#ifndef __EXTENSIONS__
+#define __EXTENSIONS__ /* This is used to get localtime_r() on Solaris. */
+#endif
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -28,6 +37,7 @@
 #include <time.h>
 #include <unistd.h>
 #include <sys/time.h>
+#include <sys/resource.h>
 #include <sys/utsname.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -64,6 +74,9 @@
 #include <io.h>
 #include <process.h>
 #include <stdlib.h>
+#include <tchar.h>
 #include <time.h>
 #include <winsock.h>
 #endif
+
+#endif /* !_SYS_INCLUDES_H_INCLUDED */
